@@ -1,19 +1,8 @@
-# encoding: latin-1
-
-# openblas + multiprocessing crashes for OPENBLAS_NUM_THREADS > 1 !!!
-import os
-os.putenv("OPENBLAS_NUM_THREADS", "1")
-
-try:
-    profile
-except NameError:
-    profile = lambda x: x
-
-import sklearn.lda
 import numpy as np
 import inspect
 
-from data_handling import Experiment
+from sklearn.discriminant_analysis import LinearDiscriminantAnalysis
+from .data_handling import Experiment
 
 
 class AbstractLearner(object):
@@ -39,7 +28,8 @@ class LinearLearner(AbstractLearner):
 
     def score(self, peaks, use_main_score):
         X = peaks.get_feature_matrix(use_main_score)
-        return np.dot(X, self.get_parameters())
+        result = np.dot(X, self.get_parameters()).astype(np.float32)
+        return result
 
     @classmethod
     def averaged_learner(clz, params):
@@ -65,7 +55,7 @@ class LDALearner(LinearLearner):
         X = np.vstack((X0, X1))
         y = np.zeros((X.shape[0],))
         y[X0.shape[0]:] = 1.0
-        classifier = sklearn.lda.LDA()
+        classifier = LinearDiscriminantAnalysis()
         classifier.fit(X, y)
         self.classifier = classifier
         self.scalings = classifier.scalings_.flatten()
