@@ -1,22 +1,83 @@
-# encoding: latin-1
-
-# openblas + multiprocessing crashes for OPENBLAS_NUM_THREADS > 1 !!!
-import os
-os.putenv("OPENBLAS_NUM_THREADS", "1")
-
-import random
-
 import pandas as pd
 import numpy as np
-from optimized import find_top_ranked, rank
-from config import CONFIG
+import random
+import click
+import sys
+import os
+
+from .optimized import find_top_ranked, rank
 
 try:
     profile
 except NameError:
-    profile = lambda x: x
+    def profile(fun):
+        return fun
 
-import logging
+
+# Parameter transformation functions
+def transform_pi0_lambda(ctx, param, value):
+    if value[1] == 0 and value[2] == 0:
+        pi0_lambda = value[0]
+    elif 0 <= value[0] < 1 and value[0] <= value[1] <= 1 and 0 < value[2] < 1:
+        pi0_lambda = np.arange(value[0], value[1], value[2])
+    else:
+        sys.exit('Error: Wrong input values for pi0_lambda. pi0_lambda must be within [0,1).')
+    return(pi0_lambda)
+
+
+def transform_threads(ctx, param, value):
+    if value == -1:
+        value = multiprocessing.cpu_count()
+    return(value)
+
+
+def transform_subsample_ratio(ctx, param, value):
+    if value < 0 or value > 1:
+      sys.exit('Error: Wrong input values for subsample_ratio. subsample_ratio must be within [0,1].')
+    return(value)
+
+
+def is_sqlite_file(filename):
+    # https://stackoverflow.com/questions/12932607/how-to-check-with-python-and-sqlite3-if-one-sqlite-database-file-exists
+    from os.path import isfile, getsize
+
+    if not isfile(filename):
+        return False
+    if getsize(filename) < 100: # SQLite database file header is 100 bytes
+        return False
+
+    with open(filename, 'rb') as fd:
+        header = fd.read(100)
+
+    if 'SQLite format 3' in str(header):
+        return True
+    else:
+        return False
+
+
+def check_sqlite_table(con, table):
+    table_present = False
+    c = con.cursor()
+    c.execute('SELECT count(name) FROM sqlite_master WHERE type="table" AND name="%s"' % table)
+    if c.fetchone()[0] == 1:
+        table_present = True
+    else:
+        table_present = False
+    c.fetchall()
+
+    return(table_present)
+
+
+def check_for_unique_blocks(tg_ids):
+    seen = set()
+    last_tg_id = None
+    for tg_id in tg_ids:
+        if last_tg_id != tg_id:
+            last_tg_id = tg_id
+            if tg_id in seen:
+                return False
+            seen.add(tg_id)
+    return True
 
 
 @profile
@@ -31,8 +92,8 @@ def cleanup_and_check(df):
     df_cleaned = df.loc[valid_rows, :]
 
     # decoy / non decoy sub tables
-    df_decoy = df_cleaned[df_cleaned["is_decoy"] == True]
-    df_target = df_cleaned[df_cleaned["is_decoy"] == False]
+    df_decoy = df_cleaned[df_cleaned["is_decoy"].eq(True)]
+    df_target = df_cleaned[df_cleaned["is_decoy"].eq(False)]
 
     # groups
     decoy_groups = set(df_decoy["tg_id"])
@@ -41,68 +102,65 @@ def cleanup_and_check(df):
     n_decoy = len(decoy_groups)
     n_target = len(target_groups)
 
-    msg = "data set contains %d decoy and %d target transition groups" % (n_decoy, n_target)
-    logging.info(msg)
+    click.echo("Info: Data set contains %d decoy and %d target groups." % (n_decoy, n_target))
     if n_decoy < 10 or n_target < 10:
-        logging.error("need at least 10 decoy groups ans 10 non decoy groups")
-        raise Exception("need at least 10 decoy groups ans 10 non decoy groups. %s" % msg)
+        sys.exit("Error: At least 10 decoy groups and 10 target groups are required.")
 
     return df_cleaned
 
 
-@profile
 def prepare_data_table(table, tg_id_name="transition_group_id",
                        decoy_name="decoy",
                        main_score_name=None,
-                       loaded_score_columns=None,
-                       **extra_args_to_dev_null
+                       score_columns=None,
                        ):
-
-    column_names = table.columns.values
-
-    if loaded_score_columns is not None:
-        missing = set(loaded_score_columns) - set(column_names)
+    N = len(table)
+    if not N:
+        sys.exit("Error: Empty input file supplied.")
+    header = table.columns.values
+    if score_columns is not None:
+        missing = set(score_columns) - set(header)
         if missing:
-            missing_txt = ", ".join("'%s'" % m for m in missing)
-            msg = "column(s) %s missing in input file for applying existing scorer" % missing_txt
-            raise Exception(msg)
+            missing_txt = ", ".join(["'%s'" % m for m in missing])
+            sys.exit("Error: Column(s) %s missing in input file to apply scorer." % missing_txt)
 
-    # check if given column names appear in table:
-    assert tg_id_name in column_names, "colum %s not in table" % tg_id_name
-    assert decoy_name in column_names, "colum %s not in table" % decoy_name
+    assert tg_id_name in header, "Error: Column %s is not in input file(s)." % tg_id_name
+    assert decoy_name in header, "Error: Column %s is not in input file(s)." % decoy_name
 
-    if loaded_score_columns is not None:
-        # we assume there is exactly one main_score in loaded_score_columns as we checked that in
+    if score_columns is not None:
+        # we assume there is exactly one main_score in score_columns as we checked that in
         # the run which persisted the classifier:
-        var_column_names = [c for c in loaded_score_columns if c.startswith("var_")]
-        main_score_name = [c for c in loaded_score_columns if c.startswith("main_")][0]
+        var_column_names = [c for c in score_columns if c.startswith("var_")]
+        main_score_name = [c for c in score_columns if c.startswith("main_")][0]
     else:
         if main_score_name is not None:
-            assert main_score_name in column_names, "colum %s not in table" % main_score_name
+            assert main_score_name in header, "Error: Column %s is not in input file(s)." % main_score_name
 
         # if no main_score_name provided, look for unique column with name
         # starting with "main_":
         else:
-            main_columns = [c for c in column_names if c.startswith("main_")]
+            main_columns = set(c for c in header if c.startswith("main_"))
             if not main_columns:
-                raise Exception("no column with main_* in table")
+                sys.exit("Error: No column \"main_*\" is in input file(s).")
             if len(main_columns) > 1:
-                raise Exception("multiple columns with name main_* in table")
-            main_score_name = main_columns[0]
+                sys.exit("Error: Multiple columns with name \"main_*\" are in input file(s).")
+            main_score_name = main_columns.pop()
 
         # get all other score columns, name beginning with "var_"
-        var_column_names = [c for c in column_names if c.startswith("var_")]
+        var_column_names = tuple(h for h in header if h.startswith("var_"))
+
         if not var_column_names:
-            raise Exception("no column with name var_* in table")
+            raise Exception("Error: No column \"var_*\" is in input file(s).")
 
     # collect needed data:
-    column_names = "tg_id tg_num_id is_decoy is_top_peak is_train main_score".split()
-    N = len(table)
     empty_col = [0] * N
     empty_none_col = [None] * N
 
     tg_ids = table[tg_id_name]
 
+    if not check_for_unique_blocks(tg_ids):
+        sys.exit("Error: " + tg_id_name + " values do not form unique blocks in input file(s).")
+
     tg_map = dict()
     for i, tg_id in enumerate(tg_ids.unique()):
         tg_map[tg_id] = i
@@ -116,16 +174,17 @@ def prepare_data_table(table, tg_id_name="transition_group_id",
                 main_score=table[main_score_name].values,
                 )
 
-    ignore_invalid_scores = CONFIG["ignore.invalid_score_columns"]
+    column_names = ["tg_id", "tg_num_id", "is_decoy", "is_top_peak", "is_train", "main_score"]
+    used_var_column_names = []
     for i, v in enumerate(var_column_names):
         col_name = "var_%d" % i
         col_data = table[v]
         if pd.isnull(col_data).all():
-            msg = "column %s contains only invalid/missing values" % v
-            if ignore_invalid_scores:
-                logging.warn("%s. pyprophet skips this.")
-                continue
-            raise Exception("%s. you may use --ignore.invalid_score_columns")
+            click.echo("Warning: Column %s contains only invalid/missing values. Column will be dropped." % v)
+            continue
+        else:
+            used_var_column_names.append(v)
+
         data[col_name] = col_data
         column_names.append(col_name)
 
@@ -135,13 +194,8 @@ def prepare_data_table(table, tg_id_name="transition_group_id",
     # build data frame:
     df = pd.DataFrame(data, columns=column_names)
 
-    all_score_columns = tuple(var_column_names) + (main_score_name,)
-
+    all_score_columns = (main_score_name,) + tuple(used_var_column_names)
     df = cleanup_and_check(df)
-
-    # for each transition group: enumerate peaks in this group, and
-    # add peak_rank where increasing rank corresponds to decreasing main
-    # score. peak_rank == 0 is peak with max main score
     return df, all_score_columns
 
 
@@ -149,13 +203,13 @@ class Experiment(object):
 
     @profile
     def __init__(self, df):
-        self.df = df
+        self.df = df.copy()
 
     def log_summary(self):
-        logging.info("summary input file:")
-        logging.info("   %d lines" % len(self.df))
-        logging.info("   %d transition groups" % len(self.df.tg_id.unique()))
-        logging.info("   %d scores including main score" % (len(self.df.columns.values) - 6))
+        click.echo("Info: Summary of input data:")
+        click.echo("Info: %d peak groups" % len(self.df))
+        click.echo("Info: %d group ids" % len(self.df.tg_id.unique()))
+        click.echo("Info: %d scores including main score" % (len(self.df.columns.values) - 6))
 
     def __getitem__(self, *args):
         return self.df.__getitem__(*args)
@@ -165,15 +219,15 @@ class Experiment(object):
 
     def __setattr__(self, name, value):
         if name not in ["df", ]:
-            raise Exception("for setting table columns use '[...]' syntax")
+            sys.exit("Error: Use '[...]' syntax to set input file columns.")
         object.__setattr__(self, name, value)
 
     def set_and_rerank(self, col_name, scores):
-        self.df[col_name] = scores
+        self.df.loc[:, col_name] = scores
         self.rank_by(col_name)
 
     def rank_by(self, score_col_name):
-        flags = find_top_ranked(self.df.tg_num_id.values, self.df[score_col_name].values)
+        flags = find_top_ranked(self.df.tg_num_id.values, self.df[score_col_name].values.astype(np.float32, copy=False))
         self.df.is_top_peak = flags
 
     def get_top_test_peaks(self):
@@ -225,9 +279,10 @@ class Experiment(object):
         target_ids = target_ids[:int(len(target_ids) * fraction) + 1]
         learn_ids = np.concatenate((decoy_ids, target_ids))
         ix_learn = df.tg_id.isin(learn_ids)
-        df.is_train[ix_learn] = True
-        df.is_train[~ix_learn] = False
+        df.loc[ix_learn,'is_train'] = True
+        df.loc[~ix_learn,'is_train'] = False
 
     def get_train_peaks(self):
         df = self.df[self.df.is_train == True]
         return Experiment(df)
+
